# Stubs for six.moves.urllib.parse
from urlparse import ParseResult as ParseResult
from urlparse import SplitResult as SplitResult
from urlparse import parse_qs as parse_qs
from urlparse import parse_qsl as parse_qsl
from urlparse import urldefrag as urldefrag
from urlparse import urljoin as urljoin
from urlparse import urlparse as urlparse
from urlparse import urlsplit as urlsplit
from urlparse import urlunparse as urlunparse
from urlparse import urlunsplit as urlunsplit
from urllib import quote as quote
from urllib import quote_plus as quote_plus
from urllib import unquote as unquote
from urllib import unquote_plus as unquote_plus
from urllib import urlencode as urlencode
from urllib import splitquery as splitquery
from urllib import splittag as splittag
from urllib import splituser as splituser
from urlparse import uses_fragment as uses_fragment
from urlparse import uses_netloc as uses_netloc
from urlparse import uses_params as uses_params
from urlparse import uses_query as uses_query
from urlparse import uses_relative as uses_relative
